package container;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;
import java.util.Observable;

/**
 * A standard account
 * 
 * @author Valerius Weigandt
 */
public class Account extends Observable implements Serializable{

	/**
	 * serialization
	 */
	private static final long serialVersionUID = -2180998046770232498L;

	/** A unique account ID */
	private int accountID;

	/** Amount of money in cent */
	private BigDecimal balance;

	/** History with all transactions which belongs to this account */
	private List<Transaction> history;

	/**
	 * @return the accountID
	 */
	public int getAccountID() {
		return accountID;
	}

	/**
	 * Will generate automatically a account with an unique ID and will set
	 * balance to zero.
	 */
	public Account() {
		setBalance(new BigDecimal(0));
	}

	/**
	 * Checks if ID is unique and set balance to zero
	 * 
	 * @param accountID
	 *            a unique accountID
	 */
	public Account(int accountID) {
		setAccountID(accountID);
		setBalance(new BigDecimal(0));
	}

	/**
	 * @param accountID
	 *            the accountID to set
	 */
	public void setAccountID(int accountID) {
		this.accountID = accountID;
	}

	/**
	 * @return the balance
	 */
	public BigDecimal getBalance() {
		return balance;
	}

	/**
	 * @param balance
	 *            the balance to set
	 */
	public void setBalance(BigDecimal balance) {
		this.balance = balance;
		notifyObservers();
	}

	/**
	 * @param history
	 *            the history to set
	 */
	public void setHistory(List<Transaction> history) {
		this.history = history;
	}

	/**
	 * @return the history
	 */
	public List<Transaction> getHistory() {
		return history;
	}
	
	@Override
	public String toString() {
		return String.valueOf(accountID);
	}
}
