package container;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * Default class for a transaction.
 * 
 * @author Valerius Weigant
 */
public class Transaction implements Serializable {
	
	/**serialtization*/
	private static final long serialVersionUID = 4574232077766074534L;

	/** The transaction ID */
	private int transID;

	/** the account where to take the money */
	private Account fromAccount;
	
	/** the account where to translate the money */
	private Account toAccount;

	/** The amount of money */
	private BigDecimal amount;

	/** The Date */
	private Date date;

	/** Reason for payment */
	private String paymentReason;

	/** Type of the transaction */
	private TransactionType type;

	/**
	 * @return the fromAccount
	 */
	public Account getFromAccount() {
		return fromAccount;
	}

	/**
	 * @param fromAccount
	 *            the fromAccount to set
	 */
	public void setFromAccount(Account fromAccount) {
		this.fromAccount = fromAccount;
	}

	/**
	 * @return the toAccount
	 */
	public Account getToAccount() {
		return toAccount;
	}

	/**
	 * @param toAccount
	 *            the toAccount to set
	 */
	public void setToAccount(Account toAccount) {
		this.toAccount = toAccount;
	}

	/**
	 * @param transID
	 * @param fromAccount
	 * @param toAccount
	 * @param amount
	 * @param type
	 * @param paymentReason
	 */
	public Transaction(int transID, Account fromAccount, Account toAccount,
			BigDecimal amount, TransactionType type, String paymentReason) {
		this.transID = transID;
		this.fromAccount = fromAccount;
		this.toAccount = toAccount;
		this.amount = amount;
		this.type = type;
		this.paymentReason = paymentReason;
	}

	/**
	 * Empty constructor
	 */
	public Transaction() {
	}

	/**
	 * @return the date of the transaction
	 */
	public Date getDate() {
		return date;
	}

	/**
	 * @param date the date when the transaction took place 
	 */
	public void setDate(Date date) {
		this.date = date;
	}

	/**
	 * @return the amount
	 */
	public BigDecimal getAmount() {
		return amount;
	}

	/**
	 * @param amount
	 *            the amount to set
	 */
	public void setAmount(BigDecimal amount) {
		this.amount = amount;
	}

	/**
	 * @param transID
	 *            the transID to set
	 */
	public void setTransID(int transID) {
		this.transID = transID;
	}

	/**
	 * @return the transID
	 */
	public int getTransID() {
		return transID;
	}

	/**
	 * @param type
	 *            the type to set
	 */
	public void setType(TransactionType type) {
		this.type = type;
	}

	/**
	 * @return the type
	 */
	public TransactionType getType() {
		return type;
	}

	/**
	 * @param paymentReason
	 *            the paymentReason to set
	 */
	public void setPaymentReason(String paymentReason) {
		this.paymentReason = paymentReason;
	}

	/**
	 * @return the paymentReason
	 */
	public String getPaymentReason() {
		return paymentReason;
	}

}
