package controller.web;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;

import container.Account;
import container.Customer;
import database.Database;

/**
 * Controller for managing accounts from a registered customer
 * 
 * @author Valerius Weigandt
 */
@ManagedBean
@SessionScoped
public class AccountController implements Serializable {
	/** Serialization */
	private static final long serialVersionUID = 1977312451792411755L;

	/** The from which to pay */
	public Account from;

	/** the customer */
	@ManagedProperty(value = "#{customerController.customer}")
	private Customer customer;

	/**
	 * @return the customer
	 */
	public Customer getCustomer() {
		return customer;
	}

	/**
	 * @param customer
	 *            the customer to set
	 */
	public void setCustomer(Customer customer) {
		this.customer = customer;
	}

	/**
	 * @return all accounts of the customer
	 */
	public List<Account> getAccounts() {
		try {
			customer.setAccounts(Database.getInstance().getAccounts(customer));
		} catch (Throwable e) {
			e.printStackTrace();
		}
		return customer.getAccounts();
	}

	/**
	 * Generates a new account
	 * 
	 * @return redirect to kontenübersicht.xhtml
	 */
	public String newAccount() {
		Account acc = new Account();
		acc.setBalance(new BigDecimal(0));
		try {
			customer.addAccount(acc);
			Database.getInstance().addAccount(acc, customer);
		} catch (Throwable e) {
			e.printStackTrace();
		}
		return "/kontenuebersicht.xhtml?faces-redirect=true";
	}

	/**
	 * @return the from
	 */
	public Account getFrom() {
		return from;
	}

	/**
	 * @param from
	 *            the from to set
	 */
	public void setFrom(Account from) {
		this.from = from;
	}

	/**
	 * @param acc
	 *            the account to use
	 * @return redirect to ueberweisung.xhtml
	 */
	public String redirect(Account acc) {
		from = acc;
		return "ueberweisung.xhtml?faces-redirect=true";
	}
}
