package controller.web;

import globals.BankSettings;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import javax.annotation.PreDestroy;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;
import javax.faces.event.ComponentSystemEvent;
import javax.validation.constraints.Pattern;

import com.sleepycat.dbxml.XmlException;

import container.Account;
import container.Customer;
import container.Gender;
import database.Database;

/**
 * Class for managing customer accounts. It handles new registrations and also logins.
 * 
 * @author Valerius Weigandt
 */
@ManagedBean
@SessionScoped
public class CustomerController implements Serializable {

	/** The Serialization */
	private static final long serialVersionUID = 994409244982161251L;

	/** the customer */
	private Customer customer;
	
	/** customer id */
	@Pattern(regexp = "[0-9]{6}|admin", message = "Bitte geben Sie 6 Zahlen ein!")
	private String customerID;
	
	/** customer pw */
	private String customerPassword;
	
	/** boolean if logged in via admin password and ID */
	private Boolean admin;

	/**
	 * @return the customer
	 */
	public Customer getCustomer() {
		return customer;
	}

	/**
	 * @param customer
	 *            the customer to set
	 */
	public void setCustomer(Customer customer) {
		this.customer = customer;
	}

//	private UIComponent findComponent(UIComponent parent, String id) {
//		if (id.equals(parent.getId())) {
//			return parent;
//		}
//		Iterator<UIComponent> kids = parent.getFacetsAndChildren();
//		while (kids.hasNext()) {
//			UIComponent kid = kids.next();
//			UIComponent found = findComponent(kid, id);
//			if (found != null) {
//				return found;
//			}
//		}
//		return null;
//	}

	/**
	 * Try to login
	 * 
	 * @return {@code null} if login fails otherwise a {@code redirect}
	 */
	public String login() {
		try {
			if (customerID.compareTo(BankSettings.adminName) == 0) {
				if (customerPassword.compareTo(BankSettings.adminPassword) == 0) {
					setAdmin(true);
					return "admin_log.xhtml?faces-redirect=true";
				} else {
					FacesMessage message = new FacesMessage("Login ist fehlgeschlagen!");
					FacesContext.getCurrentInstance().addMessage("mainForm:loginForm", message);
				}
			}
			
			customer = Database.getInstance().getCustomer(
					Integer.parseInt(customerID), customerPassword);
			if (customer != null) {
				return "/kontenuebersicht.xhtml?faces-redirect=true";
			} else {
				FacesMessage message = new FacesMessage(
						"Login ist fehlgeschlagen!");
//				FacesContext context = FacesContext.getCurrentInstance();
//				UIComponent mybutton = findComponent(context.getViewRoot(),
//						"loginButton");
//				context.addMessage(mybutton.getClientId(context), message);
//				System.out.println((mybutton.getClientId(context)));
				
				FacesContext.getCurrentInstance().addMessage("mainForm:loginForm", message);
//				UIComponent button = findComponent(FacesContext.getCurrentInstance().getViewRoot(), "loginButton");
				
			}	

		} catch (XmlException e) {
			e.printStackTrace();
		} catch (Throwable e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Generates a random customer + account (just for testing purposes)
	 */
	public void addDummyValues() {
		try {
			Customer c = new Customer();
			c.setPassword("Test");
			c.setDob(new Date());
			c.setGender(Gender.MALE);
			c.setCustomerID(Database.getInstance().addCustomer(c));

			Account a = new Account();
			a.setBalance(new BigDecimal(3000));
			Database.getInstance().addAccount(a, c);
			System.out.println(c.getCustomerID());
			System.out.println(c.getPassword());
			System.out.println(a.getAccountID());
		} catch (XmlException e) {
			e.printStackTrace();
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	/**
	 * Forward to the login page if not logged in.
	 * 
	 * @param cse
	 */
	public void checkLoggedIn(ComponentSystemEvent cse) {
		FacesContext context = FacesContext.getCurrentInstance();
		if (customer == null) {
			context.getApplication()
					.getNavigationHandler()
					.handleNavigation(context, null,
							"/login.xhtml?faces-redirect=true");
		} 
	}
	
	
	public void checkLoggedInAdmin() {
		if ((admin == null) || !admin) {
			FacesContext context = FacesContext.getCurrentInstance();
			context.getApplication()
					.getNavigationHandler()
					.handleNavigation(context, null,
							"/login.xhtml?faces-redirect=true");
		}
	}
	
	public void logoutAdmin() {
		admin = false;
		FacesContext.getCurrentInstance().getExternalContext()
			.invalidateSession();
	}
	
	/**
	 * Forward to "kontenuebersicht.xhtml" if already logged in
	 * 
	 * @param cse
	 */
	public void navigate(ComponentSystemEvent cse) {
		FacesContext context = FacesContext.getCurrentInstance();
		if (customer != null) {
			context.getApplication()
					.getNavigationHandler()
					.handleNavigation(context, null,
							"/kontenuebersicht.xhtml?faces-redirect=true");
		} 
	}
	
	/**
	 * Logout customer and redirect him to the login page
	 * 
	 * @return "/login.xhtml?faces-redirect=true"
	 */
	public String logout() {
		customer = null;
		FacesContext.getCurrentInstance().getExternalContext()
				.invalidateSession();
		return "/login.xhtml?faces-redirect=true";
	}

	/**
	 *Writes customer back to database
	 */
	public void update() {
		try {
			Database.getInstance().updateCustomer(customer);
		} catch (XmlException e) {
			e.printStackTrace();
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	/**
	 * @return the customerID
	 */
	public String getCustomerID() {
		return customerID;
	}

	/**
	 * @param customerID
	 *            the customerID to set
	 */
	public void setCustomerID(String customerID) {
		this.customerID = customerID;
	}

	/**
	 * @return the customerPassword
	 */
	public String getCustomerPassword() {
		return customerPassword;
	}

	/**
	 * @param customerPassword
	 *            the customerPassword to set
	 */
	public void setCustomerPassword(String customerPassword) {
		this.customerPassword = customerPassword;
	}

	/**
	 * Notifies the Database that this object is going to be destroyed
	 */
	@PreDestroy
	public void signOff() {
		if (customerID != null)
			if (customerID.compareTo(BankSettings.adminName) != 0)
				Database.getInstance().signOff(Integer.parseInt(customerID));
	}

	public boolean isAdmin() {
		return admin;
	}

	public void setAdmin(boolean admin) {
		this.admin = admin;
	}
}
