package controller.web;

import globals.BankSettings;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.regex.Pattern;

import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletResponse;

import jxl.Workbook;
import jxl.WorkbookSettings;
import jxl.write.Label;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;

import com.itextpdf.text.Document;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

import container.Account;
import container.Customer;
import container.Transaction;
import database.Database;

/**
 * Handler for exchanges.
 * 
 * @author Valerius W.
 */
@javax.faces.bean.ManagedBean
@ViewScoped
public class ExchangeController implements Serializable {
	
	/**serialization */
	private static final long serialVersionUID = -3965444721392583416L;

	/**
	 * {@link AccountController} of the current running session
	 */
	@ManagedProperty(value = "#{accountController}")
	private AccountController accountController;
	
	/** the account to filter */
	private String acc;
	
	/**
	 * filter from when the {@link Transaction}s should be displayed
	 */
	private Date from;
	
	/**
	 * filter until when {@link Transaction}s should be displayed
	 */
	private Date until;
	
	/**
	 * filters for a concrete payment reason
	 */
	private String paymentReason;
	
	/**
	 * Calculates the table again and reloads the side
	 * @return {@code null}
	 */
	public String refresh(){
		transactions = calculateTransactions();
		return null;
	}
	
	/**
	 * transactions to display
	 */
	private List<Transaction> transactions =null; 
	
	/**
	 * the account to filter
	 * if accID is less then 1000 then don't use it
	 * @param accID
	 */
	public void setAccount(int accID){
		if(accID > 9999){
			acc = String.valueOf(accID);
			transactions = calculateTransactions();
		}
	}
	
	/**
	 * build the {@link Transaction} list and filter also for {@link #acc}, {@link #paymentReason},{@link #from} and {@link #until} 
	 * 
	 * @return the filtered {@link Transaction} list
	 */
	public List<Transaction> calculateTransactions(){ 
		List<Account> accounts = accountController.getAccounts();
		if(accounts==null) return null;

		Account fromAcc = null;
		for (Account account : accounts) {
			if(account.toString().equals(acc)){
				fromAcc = account;
				break;
			}
		}
		
		List<Transaction> result = new ArrayList<Transaction>();
		List<Transaction> transactions;
		try {
			transactions = Database.getInstance().getTransactions(fromAcc);
		} catch (Throwable e) {
			return null;
		}
			
		if (transactions==null) return null;
		
		Pattern payment = null;
		if(paymentReason!=null){
			payment = Pattern.compile(".*"+Pattern.quote(paymentReason.toLowerCase())+".*");
		}
		
		if(from==null && until == null)
			result =  transactions;
		
		if(from!=null && until == null)
			for (Transaction transaction : transactions)
				if(transaction.getDate().after(from)) result.add(transaction);
			
		if(from!=null && until != null)
			for (Transaction transaction : transactions)
				if(transaction.getDate().after(from) && transaction.getDate().before(until)) result.add(transaction);
		
		if(from==null && until != null)
			for (Transaction transaction : transactions)
				if(transaction.getDate().before(until)) result.add(transaction);
		
		List<Transaction> endResult = new LinkedList<Transaction>();
		if(paymentReason == null){
			endResult = result;
		} else {
			for (Transaction transaction : result) {
				if(payment.matcher(transaction.getPaymentReason().toLowerCase()).matches()){
					endResult.add(transaction);
				} 
			}			
		}
		
//		for (Transaction transaction : endResult) {
//			if(transaction.getFromAccount().getAccountID() == fromAcc.getAccountID()){
//				if(transaction.getAmount().floatValue() > 0)	
//					transaction.setAmount(transaction.getAmount().negate());
//			}
//		}
		return endResult;
	}
	
	/**
	 * Exports the results to a pdf
	 */
	public void exportPdf() {
		try {
			Customer c= accountController.getCustomer();
			DateFormat format = new SimpleDateFormat("dd.MM.yyyy");
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			PdfPCell cell;
			Document document = new Document();
			PdfWriter.getInstance(document, baos);
			Font defaultFont = new Font(Font.FontFamily.HELVETICA, 10);
			Font headerFont = new Font(Font.FontFamily.HELVETICA, 10, Font.BOLD);
			document.open();
			document.add(new Paragraph("Umsatzanzeige für " + c.getFirstName()+ " " + c.getFamilyName()
					                   + ", erstellt am " + format.format(new Date())
					                   + " für Konto " + acc,defaultFont));
		
					             
//			document.add(new Paragraph("Zweiter Paragraph"));
			
			
			PdfPTable table = new PdfPTable(new float[] {1f, 4f, 1f});
			table.setWidthPercentage(100f);
			table.setSpacingBefore(20f);
			cell = new PdfPCell(new Phrase("Datum", headerFont));
			cell.setHorizontalAlignment(Element.ALIGN_CENTER);
			cell.setPadding(4f);	
			cell.setGrayFill(0.9f);
			table.addCell(cell);
			cell = new PdfPCell(new Phrase("Umsatzinformation", headerFont));
			cell.setPadding(4f);	
			cell.setGrayFill(0.9f);
			table.addCell(cell);
			cell = new PdfPCell(new Phrase("Euro", headerFont));
			cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
			cell.setPadding(4f);	
			cell.setGrayFill(0.9f);
			table.addCell(cell);
			
			if(transactions !=null)
			for (Transaction transaction: transactions) {
				cell = new PdfPCell(new Phrase(format.format(transaction.getDate()), defaultFont));
				cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
				cell.setBorder(Rectangle.BOTTOM);
				cell.setPadding(4f);	
				table.addCell(cell);
				
				if(transaction.getFromAccount().toString().equals(acc)){
					cell = new PdfPCell(new Phrase("AN : " + transaction.getToAccount() + ", " + transaction.getPaymentReason(), defaultFont));
				} else {
					cell = new PdfPCell(new Phrase("VON: " + (transaction.getFromAccount().getAccountID() == BankSettings.bankAccountID? "E-BANK-SYSTEM":transaction.getFromAccount()) + ", " + transaction.getPaymentReason(), defaultFont));
				}

				cell.setBorder(Rectangle.BOTTOM);
				cell.setPadding(4f);
				table.addCell(cell);
				cell = new PdfPCell(new Phrase(transaction.getAmount().setScale(2,BigDecimal.ROUND_HALF_UP).toString(), defaultFont));
				cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
				cell.setBorder(Rectangle.BOTTOM);
				cell.setPadding(4f);
				table.addCell(cell);
			}
			document.add(table);
			document.close();
			
			FacesContext context = FacesContext.getCurrentInstance();
			HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
			response.setContentType("application/pdf");
			response.setContentLength(baos.size());
			response.setHeader("Content-disposition", "inline;filename=\"umsaetze.pdf\"");
			OutputStream out = response.getOutputStream();
			baos.writeTo(out);
			out.flush();
			response.flushBuffer();
			context.responseComplete();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	
	/**
	 * exports the results to a excel file
	 */
	public void exportExcel() {
		try {
			FacesContext context = FacesContext.getCurrentInstance();
			HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
			response.setContentType("application/vnd.ms-excel");
			response.setHeader("Content-Disposition", "attachment; filename=\"umsaetze.xls\";");
			OutputStream out = response.getOutputStream();
			
			WorkbookSettings ws = new WorkbookSettings();
			ws.setLocale(new Locale("de", "DE"));
			WritableWorkbook workbook = Workbook.createWorkbook(out, ws);
			WritableSheet sheet = workbook.createSheet("Sheet1", 0);
			sheet.addCell(new Label(0, 0, "Datum"));
			sheet.addCell(new Label(1, 0, "Umssatzinformation"));
			sheet.addCell(new Label(2, 0, "EURO"));
			
			DateFormat format = new SimpleDateFormat("dd.MM.yyyy");
			
			int row = 1;
			if(transactions !=null)
			for (Transaction transaction: transactions) {
				sheet.addCell(new Label(0, row, format.format(transaction.getDate())));
				
				if(transaction.getFromAccount().toString().equals(acc)){
					sheet.addCell(new Label(1, row, "AN :" + transaction.getToAccount().toString() + ", " + transaction.getPaymentReason()));
				} else {
					sheet.addCell(new Label(1, row, "VON:" + (transaction.getFromAccount().getAccountID() == BankSettings.bankAccountID? "E-BANK-SYSTEM":transaction.getFromAccount()) + ", " + transaction.getPaymentReason()));
				}
				

				sheet.addCell(new Label(2, row, transaction.getAmount().setScale(2,BigDecimal.ROUND_HALF_UP).toString()));

				row++;
			}
			workbook.write();
			workbook.close();
			response.flushBuffer();
			context.responseComplete();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @return the acc
	 */
	public String getAcc() {
		return acc;
	}

	/**
	 * @param acc the acc to set
	 */
	public void setAcc(String acc) {
		this.acc = acc;
	}

	/**
	 * @return the from
	 */
	public Date getFrom() {
		return from;
	}

	/**
	 * @param from the from to set
	 */
	public void setFrom(Date from) {
		this.from = from;
	}

	/**
	 * @return the until
	 */
	public Date getUntil() {
		return until;
	}

	/**
	 * @param until the until to set
	 */
	public void setUntil(Date until) {
		this.until = until;
	}

	/**
	 * @return the paymentReason
	 */
	public String getPaymentReason() {
		return paymentReason;
	}

	/**
	 * @param paymentReason the paymentReason to set
	 */
	public void setPaymentReason(String paymentReason) {
		this.paymentReason = paymentReason;
	}

	/**
	 * @return the accountController
	 */
	public AccountController getAccountController() {
		return accountController;
	}

	/**
	 * @param accountController the accountController to set
	 */
	public void setAccountController(AccountController accountController) {
		this.accountController = accountController;
	}

	/**
	 * @return the transactions
	 */
	public List<Transaction> getTransactions() {
		return transactions;
	}

	/**
	 * @param transactions the transactions to set
	 */
	public void setTransactions(List<Transaction> transactions) {
		this.transactions = transactions;
	}
	
}
