package controller.web;

import java.io.Serializable;
import java.util.Date;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;

import container.Customer;
import container.Gender;
import database.Database;

/**
 * Handles new registrations
 * 
 * @author Carl
 * @author Valerius
 */
@ManagedBean
@ViewScoped
public class RegisterController implements Serializable {

	/** serialization */
	private static final long serialVersionUID = 1412070084281396101L;

	/** new customerID */
	private Integer customerID;

	/** first name */
	@Pattern(regexp = "[a-zA-Z]+", message = "Nur Buchstaben a-z")
	private String firstName;

	/** last name */
	@Pattern(regexp = "[a-zA-Z]+", message = "Nur Buchstaben a-z")
	private String lastName;

	// private String email;

	/** birthday */
	private Date birthday;

	/** new password */
	@Size(min = 3)
	private String password;

	// private Integer plz;
	// private String city;
	// private String street;
	// private Integer telNumber;

	/** gender of the customer */
	private Gender gender;

	/**
	 * if <code>true</code> the the registration is not over else the new
	 * {@link #customerID} will be shown
	 */
	private boolean registering = true;

	/**
	 * bakes new customer :)
	 */
	public void createCustomer() {
		Customer c = new Customer();
		c.setFirstName(firstName);
		c.setFamilyName(lastName);
		// c.seteMail(email);
		c.setDob(birthday);
		c.setPassword(password);
		// c.setPlz(plz);
		// c.setStreet(street);
		// c.setCity(city);
		// c.setTelephoneNumber(telNumber);
		if (gender.compareTo(Gender.MALE) == 0)
			c.setGender(Gender.MALE);
		else
			c.setGender(Gender.FEMALE);

		try {
			customerID = Database.getInstance().addCustomer(c);
		} catch (Throwable e) {
			e.printStackTrace();
		}

		registering = false;

	}

	/**
	 * @return the firstName
	 */
	public String getFirstName() {
		return firstName;
	}

	/**
	 * @param firstName
	 *            the {@link #firstName} to set
	 */
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	/**
	 * @return lastName
	 */
	public String getLastName() {
		return lastName;
	}

	/**
	 * @param lastName
	 *            the {@link #lastName} to set
	 */
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	// public String getEmail() {
	// return email;
	// }
	//
	// public void setEmail(String email) {
	// this.email = email;
	// }

	/**
	 * @return the {@link #birthday}
	 */
	public Date getBirthday() {
		return birthday;
	}

	/**
	 * @param birthdate
	 *            the {@link #birthday} to set
	 */
	public void setBirthday(Date birthdate) {
		this.birthday = birthdate;
	}

	/**
	 * @return the {@link #password}
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * @param password
	 *            the {@link #password} to set
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	// public int getPlz() {
	// return plz;
	// }
	//
	// public void setPlz(int plz) {
	// this.plz = plz;
	// }

	// public String getCity() {
	// return city;
	// }
	//
	// public void setCity(String city) {
	// this.city = city;
	// }

	// public String getStreet() {
	// return street;
	// }
	//
	// public void setStreet(String street) {
	// this.street = street;
	// }

	// public int getTelNumber() {
	// return telNumber;
	// }
	//
	// public void setTelNumber(int telNumber) {
	// this.telNumber = telNumber;
	// }

	/**
	 * @return the {@link #gender}
	 */
	public Gender getGender() {
		return gender;
	}

	/**
	 * @param gender
	 *            the {@link #gender} to set
	 */
	public void setGender(Gender gender) {
		this.gender = gender;
	}

	/**
	 * @return the {@link #customerID}
	 */
	public Integer getCustomerID() {
		return customerID;
	}

	/**
	 * @param customerID
	 *            the {@link #customerID} to set
	 */
	public void setCustomerID(Integer customerID) {
		this.customerID = customerID;
	}

	/**
	 * @return the registered
	 */
	public boolean isRegisterd() {
		return registering;
	}

	/**
	 * @param registerd
	 *            the registered to set
	 */
	public void setRegisterd(boolean registerd) {
		this.registering = registerd;
	}

}
