package controller.web;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.event.ComponentSystemEvent;
import javax.faces.validator.ValidatorException;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

import container.Account;
import container.Transaction;
import container.TransactionType;
import database.Database;

/**
 * Handles {@link Transaction}s
 * 
 * @author Valerius
 */
@ManagedBean
@ViewScoped
public class TransactionController implements Serializable {

	/** serialization */
	private static final long serialVersionUID = 178031810675877806L;

	/**
	 * the active {@link AccountController} for the current customer
	 */
	@ManagedProperty(value = "#{accountController}")
	private AccountController accountController;

	/** the account id from where to take the money */
	private String from;

	/**
	 * account id where to transmit the money
	 */

	@NotNull(message = "Bitte geben Sie eine Kontonummer ein!")
	@Pattern(regexp = "(\\d){5,}", message = "Bitte geben Sie mindestens 5 Zahlen ein")
	private String recieverAccountID;

	/**
	 * the reason of the payment
	 */
	@NotNull(message = "Bitte geben sie ein Verwendungszweck an!")
	private String paymentReason;

	// @Pattern(regexp = "(\\d)+([,.]\\d(\\d)?)?", message = "Ungülte Eingabe")
	/**
	 * the amount to transmit
	 */
	@Min(0)
	@NotNull(message = "Bitte geben sie ein Betrag an!")
	private BigDecimal amount;

	/**
	 * @return "weitere-ueberweisung.xhtml" if everything is OK, else
	 *         <code>null</code>
	 */
	public String transmit() {

		Transaction t = new Transaction();
		Account from = new Account();
		from.setAccountID(Integer.parseInt(this.from));
		Account to = new Account();
		to.setAccountID(Integer.parseInt(recieverAccountID));

		try {

			// no transactions to same account
			if (to.getAccountID() == from.getAccountID())
				return null;

			t.setFromAccount(from);
			t.setToAccount(new Account(Integer.parseInt(recieverAccountID)));
			t.setAmount(amount);
			t.setType(TransactionType.CREDIT);
			t.setPaymentReason(paymentReason);

			for (Account a : accountController.getAccounts()) {
				if (a.getAccountID() == from.getAccountID()) {
					from.setBalance(a.getBalance());
					break;
				}
			}

			from.setBalance(from.getBalance().add(amount.negate()));

			// if committing transaction does not result in an account with
			// negative balance
			if (from.getBalance().compareTo(new BigDecimal(0)) >= 0) {
				Database.getInstance().addTransaction(t);
			} else {
				// transaction results in an account with negative balance
				return null;
			}

			return "weitere-ueberweisung.xhtml?faces-redirect=true";
		} catch (Throwable e) {
			e.printStackTrace();
		}
		// TODO Fehlerausgabe falls was schiefgegangen ist
		return null;
	}

	/**
	 * Detect selected account
	 * 
	 * @param cse
	 *            event
	 */
	public void selectAccount(ComponentSystemEvent cse) {
		if(from!=null) return;
		if (accountController.from == null)
			if (accountController.getAccounts().size() == 0){
				from = accountController.newAccount();
			}
			else
				from = accountController.getAccounts().get(0).toString();
		else
			from = accountController.from.toString(); // setup new selection

		accountController.from = null; // clear old selection
	}

	/**
	 * @return the accounts of the current user
	 */
	@SuppressWarnings("unused")
	private List<String> getAccounts() {
		List<String> result = new ArrayList<String>(accountController
				.getAccounts().size());
		for (Account acc : accountController.getAccounts()) {
			result.add(acc.toString());
		}
		return result;
	}

//	/**
//	 * refreshes the listenerFrom every time then user changes his account where
//	 * to take the money from
//	 */
//	public void ajaxListener() {
//		listenerFrom = getFrom();
//	}
	
	
	/**
	 * @param context
	 * @param component
	 * @param value
	 * @throws ValidatorException
	 */
	public void amountValidator(FacesContext context, UIComponent component,
			Object value) throws ValidatorException {
//		System.out.println("acc val :" + getFrom());
		
		int accID =  Integer.parseInt(getFrom());
//		if(listenerFrom !=null)
//			accID = Integer.parseInt(listenerFrom);
//		else accID = Integer.parseInt(getFrom());
//		System.out.println(accID);
		for (Account account: accountController.getAccounts()) {
			if(accID!=account.getAccountID()) continue;
//			System.out.println(value.getClass());
			BigDecimal amount = (BigDecimal) value;
			if((account.getBalance().floatValue()-amount.floatValue())<0)
				throw new ValidatorException(new FacesMessage("Sie haben nicht genug Geld auf dem Konto. Ihr Kontostand ist " + account.getBalance().setScale(2, BigDecimal.ROUND_HALF_UP) + "€"));
		}
	}
	
	
//	/** actual from value */
//	private String listenerFrom;

	/**
	 * @param context
	 * @param component
	 * @param value
	 * @throws ValidatorException
	 */
	public void accountValidator(FacesContext context, UIComponent component,
			Object value) throws ValidatorException {

		// System.out.println((String) value);
		String reciever = (String) value;

		if (reciever == null)
			return;

		if (reciever.equals(getFrom())) {
			// context.addMessage(component.getClientId(context),
			// new FacesMessage("Kontonummer ist ungültig"));
			throw new ValidatorException(new FacesMessage(
					"Kontonummer ist ungültig"));
			// return;
		}

		if (reciever.matches("(\\d){5,}")) {
			try {
				try {
					if (!Database.getInstance().isAccount(
							new Account(Integer.parseInt(reciever)))) {
						// context.addMessage(component.getClientId(context),
						// new FacesMessage("Kontonummer existiert nicht"));
						throw new ValidatorException(new FacesMessage(
								"Kontonummer existiert nicht"));
					}
				} catch (Throwable e) {
					throw new ValidatorException(new FacesMessage(
							"Kontonummer existiert nicht"));
				}
			} catch (NumberFormatException e) {
				// context.addMessage(component.getClientId(context),
				// new FacesMessage("Kontonummer ist ungültig"));
				throw new ValidatorException(new FacesMessage(
						"Kontonummer existiert nicht"));
			}
		}

	}

	/**
	 * @return the from
	 */
	public String getFrom() {
		return from;
	}

	/**
	 * @param from
	 *            the from to set
	 */
	public void setFrom(String from) {
		this.from = from;
	}

	/**
	 * @return the recieverAccountID
	 */
	public String getRecieverAccountID() {
		return recieverAccountID;
	}

	/**
	 * @param recieverAccountID
	 *            the recieverAccountID to set
	 */
	public void setRecieverAccountID(String recieverAccountID) {
		this.recieverAccountID = recieverAccountID;
	}

	/**
	 * @return the paymentReason
	 */
	public String getPaymentReason() {
		return paymentReason;
	}

	/**
	 * @param paymentReason
	 *            the paymentReason to set
	 */
	public void setPaymentReason(String paymentReason) {
		this.paymentReason = paymentReason;
	}

	/**
	 * @return the amount
	 */
	public BigDecimal getAmount() {
		return amount;
	}

	/**
	 * @param amount
	 *            the amount to set
	 */
	public void setAmount(BigDecimal amount) {
		this.amount = amount;
	}

	/**
	 * @return the accountController
	 */
	public AccountController getAccountController() {
		return accountController;
	}

	/**
	 * @param accountController
	 *            the accountController to set
	 */
	public void setAccountController(AccountController accountController) {
		this.accountController = accountController;
	}
}
