package database;

import globals.InterestThread;

/**
 * Singleton implementation of the actual running DB.
 * 
 * @author Carl
 */
public class Database {

	/**
	 * The actual running DB
	 */
	private static AbstractDBController implementation;

	/**
	 * Private constructor to prevent instantiation.
	 */
	private Database() {
	}

	/**
	 * Database will be initialised on first call.
	 * 
	 * @return the current instance of the database. 
	 */
	public static AbstractDBController getInstance() {
		if (implementation == null) {
			try {
				
				// Change this to a different class implementing AbstractDBController 
				// to swap databases.
				AbstractDBController concreteDatabase = new XMLDBController();
				
				// Put a proxy infront of the database to act as a cache
				// in order to speed up loading times.
				implementation = new DatabaseProxy(concreteDatabase);
				
				// Add an observer to the proxy to log database events.
				implementation.addObserver(new DatabaseEventLogger());
				
				// Start a thread monitoring when it is time to
				// grant interest to accounts.
				Thread t = new InterestThread();
				t.start();
				
			} catch (Exception e) {
				e.printStackTrace();
			}
			return implementation;
		}
		return implementation;
	}
	
}
