package database.helpers;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * An entry of a database log.
 * 
 * @author Carl
 *
 */
public class DatabaseLogEntry implements Comparable<DatabaseLogEntry> {

	/**
	 * Date of the event.
	 */
	private Date eventTime;
	
	/**
	 * String detailing the event.
	 */
	private String eventDetails;
	
	/**
	 * Rule to transform a {@link Date} to a String.
	 */
	private static final String dateFormat = "yyyy-MM-dd'T'HH:mm:ssz";
	
	/**
	 * Creates a new {@link DatabaseLogEntry} object by parsing a String.
	 * 
	 * @param entry
	 */
	public DatabaseLogEntry(String entry) {
		DateFormat formatter = new SimpleDateFormat(dateFormat);
		try {
			int separatorIndex = 0;
			for (int i = 0; i < entry.length(); i++) {
				if (entry.charAt(i) == '\t') {
					separatorIndex = i;
					break;
				}
			}
			
			eventTime = formatter.parse(entry.substring(0, separatorIndex));
			eventDetails = entry.substring(separatorIndex+1);
		} catch (ParseException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Creates a new {@link DatabaseLogEntry}.
	 * 
	 * @param eventTime
	 * @param eventDetails
	 */
	public DatabaseLogEntry(Date eventTime, String eventDetails) {
		this.setEventTime(eventTime);
		this.setEventDetails(eventDetails);
	}

	/**
	 * @return current {@link Date} of the event.
	 */
	public Date getEventTime() {
		return eventTime;
	}

	/** 
	 * @param eventTime
	 */
	public void setEventTime(Date eventTime) {
		this.eventTime = eventTime;
	}

	/**
	 * @return details of the event.
	 */
	public String getEventDetails() {
		return eventDetails;
	}

	/**
	 * @param eventDetails
	 */
	public void setEventDetails(String eventDetails) {
		this.eventDetails = eventDetails;
	}
	
	public String toString() {
		DateFormat formatter = new SimpleDateFormat(dateFormat);
		if ((eventTime != null) && (eventDetails != null))
			return formatter.format(eventTime) + "\t" + eventDetails;
		return null;
	}

	@Override
	public int compareTo(DatabaseLogEntry otherEntry) {
		if (this.getEventTime().before(otherEntry.getEventTime())) {
			return -1;
		} else if (this.getEventTime().after(otherEntry.getEventTime())) {
			return 1;
		}
		return 0;
	}

}
