package database.helpers;

import globals.XMLDB;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import container.Account;
import container.Customer;
import container.Transaction;

/**
 * Generates XMLDB entries from objects.
 * 
 * @author Carl
 * 
 */
public class XmlStringFactory {

	/**
	 * @param c {@link Customer} object.
	 * @return String which can be put into the database.
	 */
	public static String customerXML(Customer c) {
		DateFormat df = new SimpleDateFormat(XMLDB.datePattern);
		String customer = 	
			"<customers>" +
				"<customer>" +
					"<customerID>" +
						String.valueOf(c.getCustomerID()) +
					"</customerID>" +
					"<password>" +
						c.getPassword() +
					"</password>" +
					"<eMail>" +
						c.geteMail() +
					"</eMail>"+
					"<firstName>" +
						c.getFirstName() +
					"</firstName>" +
					"<familyName>" +
						c.getFamilyName() +
					"</familyName>"+
					"<plz>" +
						String.valueOf(c.getPlz()) +
					"</plz>" +
					"<city>" +
						String.valueOf(c.getCity()) +
					"</city>" +
					"<street>" +
						String.valueOf(c.getStreet()) +
					"</street>" +
					"<tel>" +
						String.valueOf(c.getTelephoneNumber()) +
					"</tel>" +
					"<dob>" +
						df.format(c.getDob()) +
					"</dob>" +
					"<gender>" +
						c.getGender().toString() +
					"</gender>" +
				"</customer>" +
			"</customers>";
		
		return customer;
	}
	
	/**
	 * @param c {@link Account} object.
	 * @return String which can be put into the database.
	 */
	public static String accountXML(Account a, Customer c) {
		String account = 
			"<accounts>" + 
				"<account>" + 
					"<accountID>" + 
						String.valueOf(a.getAccountID()) + 
					"</accountID>" + 
					"<customerID>" + 
						String.valueOf(c.getCustomerID()) + 
					"</customerID>" + 
					"<balance>" + 
						a.getBalance().toString() + 
					"</balance>" + 
				"</account>" + 
			"</accounts>";
			
		return account;
	}

	/**
	 * @param c {@link Transaction} object.
	 * @return String which can be put into the database.
	 */
	public static String transactionXML(Transaction t) {
		DateFormat df = new SimpleDateFormat(XMLDB.datePattern);
		String transaction =
			"<transactions>" +
				"<transaction>" +
					"<transID>" +
						String.valueOf(t.getTransID()) +
					"</transID>" +
					"<date>" +
						df.format(new Date()) +
					"</date>" +
					"<fromAccountID>" +
						String.valueOf(t.getFromAccount().getAccountID()) +
					"</fromAccountID>" +
					"<toAccountID>" +
						String.valueOf(t.getToAccount().getAccountID()) +
					"</toAccountID>" +
					"<amount>" +
						t.getAmount().toString() +
					"</amount>" +
					"<type>" +
						t.getType().toString() +
					"</type>" +
					"<reason>" +
						t.getPaymentReason() +
					"</reason>" +
				"</transaction>" +
			"</transactions>";
		
		return transaction;
	
	}
}
