package database.testcases;

import globals.XMLDB;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import org.junit.Assert;
import org.junit.Test;

import container.Account;
import container.Customer;
import container.Gender;
import container.Transaction;
import container.TransactionType;

import database.Database;

/**
 * Class which tests the Berkeley XMLDB.
 * 
 * @author Carl
 * 
 */
public class QueryTest {

	private boolean verbose = true;
	
	@Test
	public void testAddAccount() {
		try {
			Account a = new Account();
			Customer c = new Customer();
			c.setCustomerID(XMLDB.startCustomerID);
			a.setAccountID(Database.getInstance().addAccount(a, c));
			if (verbose)
				System.out.println("New account for customer "
						+ c.getCustomerID() + ", account ID is "
						+ a.getAccountID());
			Assert.assertTrue(a.getAccountID() >= XMLDB.startAccountID);
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	@Test
	public void testAddCustomer() {
		Customer c = new Customer();
		String pass = "PASSWORD";
		c.setPassword(pass);
		c.setDob(new Date());
		c.setGender(Gender.MALE);
		try {
			Database.getInstance().addCustomer(c);
			if (verbose)
				System.out.println("Adding new customer " + c.getCustomerID());
			c = Database.getInstance().getCustomer(c.getCustomerID(), c.getPassword());
			Assert.assertTrue(c != null);
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	@Test
	public void testAccountExists() {
		Account c = new Account();
		c.setAccountID(XMLDB.startAccountID);
		try {
			Assert.assertTrue(Database.getInstance().isAccount(c));
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	@Test
	public void testAddTransaction() {
		Transaction t = new Transaction();
		Account from = new Account();
		Account to = new Account();

		from.setAccountID(XMLDB.startAccountID);
		to.setAccountID(XMLDB.startAccountID + 1);

		try {
			if (!Database.getInstance().isAccount(to))
				testAddAccount();

			t.setFromAccount(from);
			t.setToAccount(to);
			t.setAmount(new BigDecimal(500));
			t.setType(TransactionType.CREDIT);
			t.setPaymentReason("Test");

			Database.getInstance().addTransaction(t);

			if (verbose)
				System.out.println("Paying " + t.getAmount().toString()
						+ " from " + from.getAccountID() + " to "
						+ to.getAccountID());

		} catch (Throwable e) {
			e.printStackTrace();
		}
	}
	
	@Test
	public void testGetAccounts() {
		Customer c = new Customer();
		c.setCustomerID(XMLDB.startCustomerID);
		try {
			List<Account> accounts = Database.getInstance().getAccounts(c);
			if (verbose)
				System.out.println("Customer " + XMLDB.startCustomerID + " has " + accounts.size()
						+ " accounts:");
			for (Account a : accounts) {
				if (verbose)
					System.out.println("ID: " + a.getAccountID()
							+ ", Balance: " + a.getBalance().toString());
				Assert.assertTrue(a.getAccountID() >= XMLDB.startAccountID);
			}
		} catch (Throwable e) {
			e.printStackTrace();
		}

	}

	@Test
	public void testGetTransactions() {
		Account a = new Account();
		a.setAccountID(XMLDB.startAccountID + 1);
		try {
			List<Transaction> trans = Database.getInstance().getTransactions(a);
			if (verbose)
				System.out.println("Transactions for account "
						+ a.getAccountID() + ":");
			for (Transaction t : trans) {
				if (t.getFromAccount().getAccountID() == a.getAccountID()) {
					Assert.assertTrue(t.getAmount() != null);
					if (verbose)
						System.out.println("Deposited "
								+ t.getAmount().toString() + " to "
								+ t.getToAccount().getAccountID() + " on " + t.getDate().toString());
				} else {
					Assert.assertTrue(t.getAmount() != null);
					if (verbose)
						System.out.println("Received "
								+ t.getAmount().toString() + " from "
								+ t.getFromAccount().getAccountID() + " on " + t.getDate().toString());
				}

			}
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}
}
