package globals;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import container.InterestRate;
import container.Transaction;

/**
 * Class which contains interest settings
 * 
 * @author Carl
 *
 */
public class BankSettings {

	private static Double interest;
	private static InterestRate interestRate;
	
	/**
	 * FromAccountID used in {@link Transaction}s coming from the bank.
	 */
	public static final int bankAccountID = -1;
	
	/**
	 * Password used in the login screen to get to the admin panel.
	 */
	public static final String adminPassword = "admin";
	
	/**
	 * Name used in the login screen to get to the admin panel.
	 */
	public static final String adminName = "admin";
	
	private static String interestReason;

	private static Properties props;
	private static final String propertiesPath = "bankSettings.properties";
			
	/**
	 * @return current interest.
	 */
	public static double getInterest() {
		if (interest == null) setupValues();
		return interest;
	}

	/**
	 * @param interest
	 */
	public static void setInterest(Double interest) {
		BankSettings.interest = interest;
		storeValues();
	}

	/**
	 * @return current rate at which interest gets granted.
	 */
	public static InterestRate getInterestRate() {
		if (interestRate == null) setupValues();
		return interestRate;
	}

	/**
	 * @param interestRate
	 */
	public static void setInterestRate(InterestRate interestRate) {
		BankSettings.interestRate = interestRate;
		storeValues();
	}
		
	/**
	 * @return reason which is set on interest transactions.
	 */
	public static String getInterestReason() {
		if (interestReason == null)
			setupValues();
		interestReason = interest*100 + "% interest (Rate: " + interestRate.toString() + ")";
		return interestReason;
	}

	/**
	 * @param interestReason
	 */
	public static void setInterestReason(String interestReason) {
		BankSettings.interestReason = interestReason;
	}
	
	private static void setupValues() {
		try {
			FileInputStream fs = new FileInputStream(new File(propertiesPath));
			props = new Properties();
			props.load(fs);
			interestRate = InterestRate.valueOf(props.getProperty("interestRate", InterestRate.HOURLY.toString()));
			interest = Double.parseDouble(props.getProperty("interest", "0.02"));
			fs.close();
		} catch (FileNotFoundException e) {
			interestRate = InterestRate.HOURLY;
			interest = 0.02;
			storeValues();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private static void storeValues() {
		props = new Properties();
		props.setProperty("interestRate", interestRate.toString());
		props.setProperty("interest", interest.toString());
		try {
			props.store(new FileOutputStream(new File(propertiesPath)), "defaults");
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
}
